;coded by UEZ build 2016-08-04
;idea taken from http://rosettacode.org/wiki/Mandelbrot_set#C.23 / JS version

#AutoIt3Wrapper_Compile_Both=y
#pragma compile(Icon, "c:\Program Files (x86)\AutoIt3\Icons\au3.ico")
#AutoIt3Wrapper_Run_Au3Stripper=y
#Au3Stripper_Parameters=/so /pe /rm
#AutoIt3Wrapper_Run_After=del /f /q "%scriptdir%\%scriptfile%_stripped.au3"

#include <GUIConstantsEx.au3>
#include <WinAPIGdi.au3>
#include <WindowsConstants.au3>

AutoItSetOption("MouseCoordMode", 2)
AutoItSetOption("MustDeclareVars", 1)

Global $hGUI, $bExit
Global Const $iW = 1000, $iH = 600, $sTitle = "GDI Mandelbrot / Took "


AutoItSetOption("GUIOnEventMode", 1)
Global $iDetails = 100, $iDisplayFrequency = 100

Mandelbrot(-2, 1, -1, 1, $iDetails, $iDisplayFrequency)


AutoItSetOption("GUIOnEventMode", 0)


Func Mandelbrot($xmin, $xmax, $ymin, $ymax, $iterations, $iDisplayFrequency = 100)
	$iDisplayFrequency = $iDisplayFrequency < 1 ? 1 : $iDisplayFrequency > $iH / 2 ? $iH / 2 : $iDisplayFrequency

	$hGUI = GUICreate($sTitle & "...", $iW, $iH)
	Local Const $hDC = _WinAPI_GetDC($hGUI)
	Local $pBits, $tBIV5HDR = DllStructCreate($tagBITMAPV5HEADER)

	With $tBIV5HDR
		.bV5Size = DllStructGetSize($tBIV5HDR)
		.bV5Width = $iW
		.bV5Height = -$iH
		.bV5Planes = 1
		.bV5BitCount = 32
		.bV5Compression = $BI_BITFIELDS
		.bV5RedMask = 0x00FF0000
		.bV5GreenMask = 0x0000FF00
		.bV5BlueMask = 0x000000FF
		.bV5AlphaMask = 0xFF000000
	EndWith

	Local Const $hBitmapGDI = _WinAPI_CreateDIBSection($hDC, $tBIV5HDR, $DIB_RGB_COLORS, $pBits)
	Global $tBitmap = DllStructCreate("uint px[" & $iW * $iH & "]", $pBits)

	Local Const $hGfxDC = _WinAPI_CreateCompatibleDC($hDC)
	Local Const $hObjOld = _WinAPI_SelectObject($hGfxDC, $hBitmapGDI)
	GUISetState(@SW_SHOW, $hGUI)

	$bExit = False
	GUISetOnEvent($GUI_EVENT_CLOSE, "_Exit")

	Local $iX, $iY, $i, $ppos, $c, $x, $y

	Local $fEnd, $fTimer = TimerInit()

	For $iY = 0 To $iH - 1
		For $iX = 0 To $iW - 1
			$x = $xmin + ($xmax - $xmin) * $iX / ($iW - 1)
			$y = $ymin + ($ymax - $ymin) * $iY / ($iH - 1)
			$i = MandelIter($x, $y, $iterations)
			$ppos = ($iW * $iY + $iX)

			If ($i > $iterations) Then
				$tBitmap.px(($ppos)) = 0xFF000000
			Else
				$c = 3 * Log($i) / Log($iterations - 1.0)
				If ($c < 1) Then
					$tBitmap.px(($ppos)) = 0xFF000000 + BitShift(Int(255 * $c), -16)
				ElseIf ($c < 2) Then
					$tBitmap.px(($ppos)) = 0xFF000000 + BitShift(255, -16) + BitShift(Int(255 * ($c - 1)), -8)
				Else
					$tBitmap.px(($ppos)) = 0xFFFFFF00 + (Int(255 * ($c - 2)))
				EndIf
			EndIf
		Next
		If Not Mod($iY, $iDisplayFrequency) Then
			_WinAPI_BitBlt($hDC, 0, 0, $iW, $iH, $hGfxDC, 0, 0, $SRCCOPY)
			ConsoleWrite(Round($iY / $iH * 100, 2) & "%" & @CRLF)
		EndIf
	Next
	_WinAPI_BitBlt($hDC, 0, 0, $iW, $iH, $hGfxDC, 0, 0, $SRCCOPY)

	$fEnd = TimerDiff($fTimer)

	WinSetTitle($hGUI, "", $sTitle & $fEnd / 1000 & " seconds to render.")

	Do
		If $bExit Then ExitLoop
	Until Not Sleep(100)

	_WinAPI_SelectObject($hGfxDC, $hObjOld)
	_WinAPI_ReleaseDC($hGUI, $hDC)
	_WinAPI_DeleteDC($hGfxDC)
	_WinAPI_DeleteObject($hBitmapGDI)
	GUIDelete($hGUI)
EndFunc   ;==>Mandelbrot

Func _Exit()
	$bExit = True
EndFunc   ;==>_Exit

Func MandelIter($cx, $cy, $maxIter)
	Local $x = 0.0, $y, $xx, $yy, $xy, $i = $maxIter
	While ($xx + $yy <= 4 And $i > 0)
		$xy = $x * $y
		$xx = $x * $x
		$yy = $y * $y
		$x = $xx - $yy + $cx
		$y = $xy + $xy + $cy
		$i -= 1
	WEnd
	Return $maxIter - $i
EndFunc   ;==>MandelIter
